﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.CommonBot;
using Styx.Common;
using System.Threading;
using System.IO;
using System.Diagnostics;
using System.Windows;
using Styx.Helpers;
using System.Reflection;
using Styx.CommonBot.Routines;
using Styx;

namespace BuddyStatsWoWPlugin
{
    class BuddyStats : HBPlugin
    {
        private int version = 0100;
        private bool isRunning;
        private Thread mainThread;
        private DateTime lastSent;
        private Connection connection;
        private bool botRunning;
        private Updater updater;
        private bool eventsSubscribed = false;

        public string ApiKey { get; set; }
        public bool IsPrivate { get; set; }

        public override string Author { get { return "Guga & Creedo"; } }
        public override string Name { get { return "BuddyStats"; } }
        public override Version Version { get { return new Version(version / 1000, version % 1000 / 10); } }
        public int VersionRaw { get { return version; } }
        public BuddyStats()
        {
            /*updater = new Updater("http://146.255.27.116/get/hb", GlobalSettings.Instance.PluginsPath, version);

            if (CheckForUpdates())
                return;
            */
            if (!Connection.LoadLibrary())
            {
                LogMessage("Loading of DLL failed! Please make sure, that file .dll files are located in Plugins/BuddyStats");
            }
        }

        public override void OnEnable()
        {
            if (!eventsSubscribed)
            {
                Styx.CommonBot.BotEvents.OnBotStarted += OnBotStart;
                Styx.CommonBot.BotEvents.OnBotStopped += OnBotStop;
                Styx.CommonBot.Chat.Whisper += OnWhisper;
                Styx.CommonBot.Chat.Guild += OnGuildMsg;
                Styx.CommonBot.Chat.Battleground += OnBattlegroundMsg;
                Styx.CommonBot.Chat.BattlegroundLeader += OnBattlegroundLeaderMsg;
                Styx.CommonBot.Chat.Raid += OnRaidMsg;
                Styx.CommonBot.Chat.RaidLeader += OnRaidLeaderMsg;
                Styx.CommonBot.Chat.Say += OnSay;
                Styx.CommonBot.Chat.Yell += OnYell;
                System.Windows.Application.Current.MainWindow.Closing += onExit;
                eventsSubscribed = true;
            }

            IsPrivate = false;
            botRunning = false;
            ApiKey = "";
            lastSent = DateTime.Now;

            connection = new Connection(this);
            LoadConfig();
            if (ApiKey == "")
            {
                LogMessage("Please register on our website and fill in your ApiKey in the Settings.cfg located in plugin directory");
                return;
            }

            LogMessage("Connecting to server and logging in...");
            connection.Connect();
            if (!connection.IsConnected())
            {
                connection.Disconnect();
                return;
            }
            LogMessage("Connected to server");

            mainThread = new Thread(MainThreadProc);
            mainThread.Start();

            LogMessage("Website:  hb.buddystats.com  ");
            LogMessage("[ON]");

            base.OnEnable();
        }

        public override void OnDisable()
        {
            if (eventsSubscribed)
            {
                Styx.CommonBot.BotEvents.OnBotStarted -= OnBotStart;
                Styx.CommonBot.BotEvents.OnBotStopped -= OnBotStop;
                Styx.CommonBot.BotEvents.OnBotStopped -= OnBotStop;
                Styx.CommonBot.Chat.Whisper -= OnWhisper;
                Styx.CommonBot.Chat.Guild -= OnGuildMsg;
                Styx.CommonBot.Chat.Battleground -= OnBattlegroundMsg;
                Styx.CommonBot.Chat.BattlegroundLeader -= OnBattlegroundLeaderMsg;
                Styx.CommonBot.Chat.Raid -= OnRaidMsg;
                Styx.CommonBot.Chat.RaidLeader -= OnRaidLeaderMsg;
                Styx.CommonBot.Chat.Say -= OnSay;
                Styx.CommonBot.Chat.Yell -= OnYell;
                System.Windows.Application.Current.MainWindow.Closing -= onExit;
                eventsSubscribed = false;
            }

            connection.Disconnect();
            if (mainThread != null && mainThread.IsAlive)
                mainThread.Abort();
            LogMessage("[OFF]");

            base.OnDisable();
        }

        public void onExit(object o, EventArgs args)
        {
            connection.Disconnect();
            if (mainThread != null && mainThread.IsAlive)
                mainThread.Abort();
        }

        private void OnBotStart(EventArgs args)
        {
            botRunning = true;
        }

        private void OnBotStop(EventArgs args)
        {
            botRunning = false;
        }
        #region ChatEvents
        private void OnWhisper(Chat.ChatWhisperEventArgs args)
        {
            connection.SendChatMessage("WHISPER", args.Author, args.Message);
        }
        private void OnGuildMsg(Chat.ChatGuildEventArgs args)
        {
            connection.SendChatMessage("GUILD", args.Author, args.Message);
        }
        private void OnBattlegroundMsg(Chat.ChatLanguageSpecificEventArgs args)
        {
            connection.SendChatMessage("BATTLEGROUND", args.Author, args.Message);
        }
        private void OnBattlegroundLeaderMsg(Chat.ChatLanguageSpecificEventArgs args)
        {
            connection.SendChatMessage("BATTLEGROUND_LEADER", args.Author, args.Message);
        }
        private void OnRaidMsg(Chat.ChatLanguageSpecificEventArgs args)
        {
            connection.SendChatMessage("RAID", args.Author, args.Message);
        }
        private void OnRaidLeaderMsg(Chat.ChatLanguageSpecificEventArgs args)
        {
            connection.SendChatMessage("RAID_LEADER", args.Author, args.Message);
        }
        private void OnSay(Chat.ChatLanguageSpecificEventArgs args)
        {
            connection.SendChatMessage("SAY", args.Author, args.Message);
        }
        private void OnYell(Chat.ChatLanguageSpecificEventArgs args)
        {
            connection.SendChatMessage("YELL", args.Author, args.Message);
        }
        #endregion


        public void SendChatMessage(string msg, string chatType, string channel)
        {
            Lua.DoString("SendChatMessage('{0}', '{1}', nil, '{2}');",msg, chatType, channel);
        }

        /* public void onExit(object o, EventArgs e)
        {
            connection.Disconnect();
            if (mainThread != null && mainThread.IsAlive)
                mainThread.Abort();
        }*/

        public string[] GetProfileList()
        {
            try
            {
                //string path = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/Profiles/";
                string path = Path.GetDirectoryName(Styx.CommonBot.Profiles.ProfileManager.XmlLocation);
                string[] profileList = Directory.GetFiles(path, "*.xml");
                string temp = profileList[0];
                profileList[0] = Styx.CommonBot.Profiles.ProfileManager.XmlLocation;
                for (int i = 1; i < profileList.Length; i++)
                {
                    if (profileList[i] == Styx.CommonBot.Profiles.ProfileManager.XmlLocation)
                        profileList[i] = temp;
                }
                return profileList;
            }
            catch (Exception e)
            {
                LogMessage("Error while loading profile: " + e.Message);
                return null;
            }
        }

        public void LoadProfileByIndex(int index)
        {
            string profilePath = GetProfileList()[index];
            Styx.CommonBot.BotManager.Current.Stop();
            Styx.CommonBot.Profiles.ProfileManager.LoadNew(profilePath);

            LogMessage("Profile [" + profilePath + "] has been loaded!");
            Thread.Sleep(1000);
            Styx.CommonBot.BotManager.Current.Start();
        }

        public override void Pulse()
        {
            if (!Styx.StyxWoW.IsInGame || !Styx.StyxWoW.IsInWorld || !connection.IsConnected())
                return;

            if (DateTime.Now.Subtract(lastSent).TotalSeconds > 10)
            {
                bool success = false;
                if (connection.IsConnected())
                {
                    success = connection.SendPlayerData();
                }
                if (!success)
                    connection.Disconnect();

                lastSent = DateTime.Now;
            }
        }
        public void MainThreadProc()
        {
            DateTime lastSentAlive = DateTime.Now;
            DateTime lastConnect = DateTime.Now;
            //int connTriesCount = 0;
            while (true)
            {
                if (!connection.IsConnected())
                {
                    if (DateTime.Now.Subtract(lastConnect).TotalSeconds > 10)
                    {
                        /*if (connTriesCount > 10)
                        {
                            if (!updater.IsUpToDate())
                            {
                                RestartDB();
                                return;
                            }
                            connTriesCount = 0;
                        }*/
                        LogMessage("Trying to reconnect...");
                        connection.Disconnect();
                        connection.Connect();
                        if (connection.IsConnected())
                            LogMessage("Reconnecting has been succesful");
                        lastConnect = DateTime.Now;
                    }
                }
                else if ((!Styx.CommonBot.TreeRoot.IsRunning) && DateTime.Now.Subtract(lastSentAlive).TotalSeconds > 30)
                {
                    try
                    {
                        if (!connection.SendIsAlive())
                            connection.Disconnect();
                        lastSentAlive = DateTime.Now;
                    }
                    catch (Exception e)
                    {
                        LogMessage("ERROR IN MAIN THREAD:" + e.StackTrace);
                    }
                }
                Thread.Sleep(1000);
            }
        }
        public bool CheckForUpdates()
        {
            LogMessage("Checking for updates...");
            try
            {
                if (!updater.IsUpToDate())
                {
                    LogMessage("New version of BuddyStats found, downloading...");
                    updater.DownloadFiles();
                    LogMessage("BuddyStats succesfuly updated, restarting plugin...");
                    //Thread t = new Thread(ReloadPluginsProc);
                    //t.Start();
                    return true;
                }
                else
                    LogMessage("Your plugin is up to date!");
            }
            catch (Exception e)
            {
                LogMessage("Checking for updates failed! ERROR:" + e.Message + "  " + e.StackTrace);
            }
            return false;
        }
        //public void ReloadPluginsProc()
        //{
        //    string[] plugins = Styx.Plugins.PluginManager.Plugins.ToArray();
        //    Styx.Common.Plugins.PluginManager.ReloadAllPlugins(GlobalSettings.Instance.PluginsPath);
        //    Styx.Common.Plugins.PluginManager.SetEnabledPlugins(plugins);
        //}
        public void RestartDB()
        {
            /*if (!CharacterSettings.Instance.EnabledPlugins.Contains("BuddyStats"))
            {
                CharacterSettings.Instance.EnabledPlugins.Add("BuddyStats");
                CharacterSettings.Instance.Save();
            }*/
            Process.Start(Assembly.GetEntryAssembly().Location, "-autostart -routine=\"" + RoutineManager.Current.Name + "\"");
            SafeCloseProcess();
        }
        public void SafeCloseProcess()
        {
            if (Thread.CurrentThread != Application.Current.Dispatcher.Thread)
            {
                Application.Current.Dispatcher.Invoke(new Action(SafeCloseProcess));
                return;
            }
            Application.Current.Shutdown();
        }
        public void LoadConfig()
        {
            if (!File.Exists("Plugins/BuddyStats/Settings.cfg"))
                return;
            StreamReader reader = new StreamReader(GlobalSettings.Instance.PluginsPath + "/BuddyStats/Settings.cfg");
            while (!reader.EndOfStream)
            {
                string[] split = reader.ReadLine().Split('=');
                if (split != null)
                {
                    if (split[0] == "IsPrivate")
                    {
                        IsPrivate = Convert.ToBoolean(split[1]);
                    }
                    else if (split[0] == "ApiKey")
                    {
                        ApiKey = split[1];
                    }
                }
            }
            reader.Close();
            LogMessage("Settings Loaded");
        }
        public void LogMessage(string msg)
        {
            Logging.Write("[BuddyStats]: " + msg);
        }
    }
}
