﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Diagnostics;
using System.IO;
using System.Drawing.Imaging;
using System.Drawing;
using System.Net.Sockets;
using System.Net;
using System.Reflection;
using Styx.Common;
using Styx;
using Styx.Helpers;

namespace BuddyStatsWoWPlugin
{
    public enum LoginResult
    {
        WrongApiKey = 0,
        Success = 1,
        WrongClientVersion = 2,
        ConnectionError = 3,
        Unknown = 4
    }
    class Connection
    {
        private TcpClient connection;
        private Thread listenThread;
        private object sendLock;
        private BuddyStats plugin;
        private bool isConnected;
        private bool isLoggedIn;
        private object aliveLock;
        #region Assembly
        private static Assembly dll;
        private static Type typePacket;
        private static Type typePacketLogin;
        private static Type typePacketPlayerData;
        private static Type typePacketScreenShot;
        private static Type typePacketIsAlive;
        private static Type typePacketProfile;
        private static Type typePacketChatMessage;

        private static ConstructorInfo conPacketLogin;
        private static ConstructorInfo conPacketPlayerData;
        private static ConstructorInfo conPacketScreenShot;
        private static ConstructorInfo conPacketIsAlive;
        private static ConstructorInfo conPacketProfile;
        private static ConstructorInfo conPacketChatMessage;

        private static MethodInfo methodPacketToBytes;
        private static MethodInfo methodBytesToPacket;
        //Packet
        private static FieldInfo fieldSessionID;
        //PacketLogin
        private static FieldInfo fieldLoginSuccess;
        #endregion
        private string serverAddress = "buddystats.com";
        //private string serverAddress = "127.0.0.1";
        private int serverPort = 16000;
        private int sessionID = 0;

        public Connection(BuddyStats plugin)
        {
            this.plugin = plugin;
            isConnected = false;
            isLoggedIn = false;
            connection = null;
            listenThread = null;
            sendLock = new object();
            aliveLock = new object();
        }
        public static bool LoadLibrary()
        {
            try
            {
                AppDomain.CurrentDomain.AssemblyResolve += ResolveAssembly;
                string path = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/" + GlobalSettings.Instance.PluginsPath + "/BuddyStats/WebStatsPackets.dll";
                dll = Assembly.LoadFile(path);
                typePacket = dll.GetType("WebStatsPackets.Packet");
                typePacketLogin = dll.GetType("WebStatsPackets.PacketLogin");
                typePacketScreenShot = dll.GetType("WebStatsPackets.PacketScreenShot");
                typePacketIsAlive = dll.GetType("WebStatsPackets.PacketIsAlive");
                typePacketProfile = dll.GetType("WebStatsPackets.PacketProfile");


                conPacketLogin = typePacketLogin.GetConstructor(new[] { typeof(int), typeof(string), typeof(int) });
                conPacketScreenShot = typePacketScreenShot.GetConstructor(new[] { typeof(int), typeof(byte[]) });
                conPacketIsAlive = typePacketIsAlive.GetConstructor(new[] { typeof(int) });
                conPacketProfile = typePacketProfile.GetConstructor(new[] { typeof(int), typeof(string[]) });

                methodPacketToBytes = typePacket.GetMethod("PacketToBytes");
                methodBytesToPacket = typePacket.GetMethod("BytesToPacket");

                fieldSessionID = typePacket.GetField("sessionID");
                fieldLoginSuccess = typePacketLogin.GetField("loginResult");


                //BuddyStatsWoWPackets.dll
                path = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/" + GlobalSettings.Instance.PluginsPath + "/BuddyStats/BuddyStatsWoWPackets.dll";

                dll = Assembly.LoadFile(path);

                typePacketPlayerData = dll.GetType("BuddyStatsWoWPackets.PacketPlayerDataWoW");
                typePacketChatMessage = dll.GetType("BuddyStatsWoWPackets.PacketChatMessage");
                                                                                //sessionID      class         level       xpTotal      xpPerHour    deaths       gold          loots     honorGained  honorPerHour  bgsCompleted   bgsWon      botType        profileName     isPrivate
                conPacketPlayerData = typePacketPlayerData.GetConstructor(new[] { typeof(int), typeof(byte), typeof(int), typeof(int), typeof(int), typeof(int), typeof(int), typeof(int), typeof(int), typeof(int), typeof(int), typeof(int), typeof(string), typeof(string), typeof(bool)});
                conPacketChatMessage = typePacketChatMessage.GetConstructor(new[] { typeof(int), typeof(string), typeof(string), typeof(string) });

                AppDomain.CurrentDomain.AssemblyResolve -= ResolveAssembly;
                return true;
            }
            catch (Exception e)
            {
                AppDomain.CurrentDomain.AssemblyResolve -= ResolveAssembly;
                Logging.Write("DLL LOADING ERROR " + e.Message + "   " + e.StackTrace);
                return false;
            }
        }
        public static Assembly ResolveAssembly(object sender, ResolveEventArgs args)
        {
            if (args.Name.Contains("WebStatsPackets"))
            {
                string path = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/" + GlobalSettings.Instance.PluginsPath + "/BuddyStats/WebStatsPackets.dll";
                return Assembly.LoadFile(path);
            }
            return null;
        }
        public void Connect()
        {
            try
            {
                connection = new TcpClient();
                connection.Connect(new IPEndPoint(Dns.GetHostAddresses(serverAddress)[0], serverPort));
                isConnected = true;
                connection.ReceiveTimeout = 2000;
                Login();
                connection.ReceiveTimeout = 0;
                listenThread = new Thread(ListenProc);
                listenThread.Start();
            }
            catch (Exception e)
            {
                Logging.Write("CONNECTION ERROR: " + e.Message + "   " + e.StackTrace);
                isConnected = false;
            }
        }
        public void Disconnect()
        {
            if (listenThread != null)
                listenThread.Abort();
            connection.Close();
            isConnected = false;
            isLoggedIn = false;
        }
        public bool IsConnected()
        {
            return isConnected && isLoggedIn;
        }
        public bool SendPlayerData()
        {
            try
            {
                object playerDataPacket = conPacketPlayerData.Invoke(new object[] 
                {
                    sessionID, 
                    (byte)StyxWoW.Me.Class, 
                    (int)StyxWoW.Me.Level, 
                    (int)StyxWoW.Me.Experience,
                    (int)Styx.CommonBot.GameStats.XPPerHour, 
                    (int)Styx.CommonBot.GameStats.Deaths, 
                    (int)StyxWoW.Me.Gold,
                    (int)Styx.CommonBot.GameStats.Loots, 
                    (int)Styx.CommonBot.GameStats.HonorGained, 
                    (int)Styx.CommonBot.GameStats.HonorPerHour,
                    (int)Styx.CommonBot.GameStats.BGsCompleted,
                    (int)Styx.CommonBot.GameStats.BGsWon,
                    Styx.CommonBot.BotManager.Current.Name,
                    Styx.CommonBot.Profiles.ProfileManager.CurrentProfile.Name,
                    plugin.IsPrivate 
                });
                byte[] buffer = (byte[])methodPacketToBytes.Invoke(null, new object[] { playerDataPacket });
                Send(buffer);
                return true;
            }
            catch (Exception e)
            {
                isConnected = false;
                return false;
            }
        }
        public bool SendChatMessage(string channel, string senderName, string message)
        {
            try
            {
                object whisperPacket = conPacketChatMessage.Invoke(new object[] { sessionID, channel, senderName, message });
                byte[] buffer = (byte[])methodPacketToBytes.Invoke(null, new object[] { whisperPacket });
                Send(buffer);
                return true;
            }
            catch (Exception e)
            {
                isConnected = false;
                return false;
            }
        }
        public bool SendScreenShot()
        {
            try
            {
                Process p = Process.GetProcessById(Styx.StyxWoW.Memory.Process.Id);
                MemoryStream stream = new MemoryStream();
                Bitmap bitmap = ScreenShotTaker.TakeScreenShot(p.MainWindowHandle);
                bitmap.Save(stream, ImageFormat.Jpeg);
                byte[] screenshot = stream.ToArray();
                stream.Close();
                bitmap.Dispose();
                object packetScreenShot = conPacketScreenShot.Invoke(new object[] { sessionID, screenshot });
                byte[] buffer = (byte[])methodPacketToBytes.Invoke(null, new object[] { packetScreenShot });
                Send(buffer);
                plugin.LogMessage("Sending screenshot... (" + (buffer.Length - 4) + " bytes)");
                return true;
            }
            catch (Exception e)
            {
                plugin.LogMessage(e.Message + "   " + e.StackTrace);
                isConnected = false;
                return false;
            }
        }
        public bool SendProfiles()
        {
            try
            {
                string[] profiles = plugin.GetProfileList();
                for (int i = 0; i < profiles.Length; i++)
                    profiles[i] = Path.GetFileName(profiles[i]);

                object profilePacket = conPacketProfile.Invoke(new object[] { sessionID, profiles });
                byte[] buffer = (byte[])methodPacketToBytes.Invoke(null, new object[] { profilePacket });
                Send(buffer);
                return true;
            }
            catch (Exception e)
            {
                return false;
            }
        }
        public bool Login()
        {
            LoginResult result = LoginResult.Unknown;
            try
            {
                object loginPacket = conPacketLogin.Invoke(new object[] { sessionID, plugin.ApiKey, plugin.VersionRaw });
                byte[] buffer = (byte[])methodPacketToBytes.Invoke(null, new object[] { loginPacket });
                // connection.GetStream().Flush();
                Send(buffer);
                connection.Client.Receive(buffer);
                object recvPacket = methodBytesToPacket.Invoke(null, new object[] { buffer });
                sessionID = (int)fieldSessionID.GetValue(recvPacket);
                result = (LoginResult)fieldLoginSuccess.GetValue(recvPacket);
            }
            catch (Exception e)
            {
                isConnected = false;
                isLoggedIn = false;
                sessionID = 0;
                result = LoginResult.ConnectionError;
            }
            HandleLoginResult(result);
            return isLoggedIn;
        }
        public bool SendIsAlive()
        {
            lock (aliveLock)
            {
                try
                {
                    object alivePacket = conPacketIsAlive.Invoke(new object[] { sessionID });
                    byte[] buffer = (byte[])methodPacketToBytes.Invoke(null, new object[] { alivePacket });
                    Send(buffer);
                    return true;
                }
                catch (Exception e)
                {
                    return false;
                }
            }
        }
        public void Send(byte[] buffer)
        {
            lock (sendLock)
            {
                connection.GetStream().Write(buffer, 0, buffer.Length);
            }
        }
        public void HandleLoginResult(LoginResult result)
        {
            isLoggedIn = false;
            if (result == LoginResult.Success)
            {
                isLoggedIn = true;
            }
            else if (result == LoginResult.WrongClientVersion)
            {
                //MessageBox.Show("Wrong Plugin Version! Please update your plugin");
                plugin.LogMessage("Wrong Plugin Version! Please update your plugin");
            }
            else if (result == LoginResult.WrongApiKey)
            {
                //MessageBox.Show("Wrong API Key!");
                plugin.LogMessage("Wrong API Key");
            }
            else
            {
                plugin.LogMessage("Server not responding!");
            }
        }
        private void ListenProc()
        {
            while (true)
            {
                string msg = "";
                Thread.Sleep(50);
                if (IsConnected())
                {
                    try
                    {
                        byte[] buffer = new byte[2048];
                        connection.Client.Receive(buffer);
                        msg = new ASCIIEncoding().GetString(buffer);
                        HandleWebRequest(msg);
                    }
                    catch (Exception e)
                    {
                        // plugin.LogMessage("Listener ERROR   " + e.Message + "    " + e.StackTrace);
                        isConnected = false;
                        isLoggedIn = false;
                        return;
                    }
                }
            }
        }
        private void HandleWebRequest(string request)
        {
            string[] split = request.Split(';');
            if (split.Length < 2)
                return;

            string cmd = split[0];
            int sessionID = Convert.ToInt32(split[1]);
            if (cmd == "COMMAND_START")
            {
                if (!Styx.CommonBot.TreeRoot.IsRunning)
                {
                    plugin.LogMessage("Start command from BuddyStats received! Starting the bot...");
                    Styx.CommonBot.BotManager.Current.Start();
                }
            }
            else if (cmd == "COMMAND_STOP")
            {
                if (Styx.CommonBot.TreeRoot.IsRunning)
                {
                    plugin.LogMessage("Stop command from BuddyStats received! Stopping the bot...");
                    Styx.CommonBot.BotManager.Current.Stop();
                    SendIsAlive();
                }
            }
            else if (cmd == "COMMAND_SCREENSHOT")
            {
                plugin.LogMessage("Screenshot command from BuddyStats received! Sending screenshots...");
                SendScreenShot();
            }
            else if (cmd == "COMMAND_PROFILE_GET")
            {
                plugin.LogMessage("Profile request from BuddyStats received! Sending profile list...");
                SendProfiles();
            }
            else if (cmd == "COMMAND_PROFILE_LOAD" && split.Length >= 3)
            {
                plugin.LogMessage("Profile load request from BuddyStats received! Loading new profile...");
                plugin.LoadProfileByIndex(Convert.ToInt32(split[2]));
            }
            else if (cmd == "COMMAND_SEND_MESSAGE" && split.Length >= 5)
            {
                plugin.LogMessage("Chat messafe command from BuddyStats received! Sending chat message...");
                plugin.SendChatMessage(split[2], split[3], split[4]);
            }
        }
    }
}
